#include "timer_one.h"

unsigned int pwm_period;
uint8_t clock_select_bits;
uint8_t old_SREG;

void (*isrCallback)();

/* interrupt service routine that
 */
ISR(TIMER1_OVF_vect)          
{
  isrCallback();
}


void timer_set_period(long microseconds)	
{
	long cycles = (F_CPU / 2000000) * microseconds;
	if (cycles < RESOLUTION) {
		clock_select_bits = _BV(CS10);
	} else if ((cycles >>= 3) < RESOLUTION) {
		clock_select_bits = _BV(CS11);
	} else if ((cycles >>= 3) < RESOLUTION) {
		clock_select_bits = _BV(CS11) | _BV(CS10);
	} else if ((cycles >>= 2) < RESOLUTION) {
		clock_select_bits = _BV(CS12);
	} else if ((cycles >>= 2) < RESOLUTION) {
		clock_select_bits = _BV(CS12) | _BV(CS10);
	} else {
		cycles = RESOLUTION - 1;
		clock_select_bits = _BV(CS12) | _BV(CS10);
	} 
	old_SREG = SREG;
	cli();
	ICR1 = pwm_period;
	SREG = old_SREG;
	
	TCCR1B &= ~(_BV(CS10) | _BV(CS11) | _BV(CS12));
	TCCR1B |= clock_select_bits;
}

void timer_init(long microseconds) {
  TCCR1A = 0;                 // clear control register A 
  TCCR1B = _BV(WGM13);        // set mode 8: phase and frequency correct pwm, stop the timer
  timer_set_period(microseconds);
}


void timer_set_pwm(uint8_t pin, int duty) {

	unsigned long dutyCycle = pwm_period;
	
	dutyCycle *= duty;
	dutyCycle >>= 10;

	old_SREG = SREG;
	cli();
	if(pin == PD5)      
		OCR1A = dutyCycle;
	else if(pin == PD4) 
		OCR1B = dutyCycle;
	SREG = old_SREG;
}

void timer_disable_pwm(uint8_t pin) {
	if(pin == PD5)       
		TCCR1A &= ~_BV(COM1A1);   
	else if(pin == PD4)
		TCCR1A &= ~_BV(COM1B1);
}

void timer_resume() {
	TCCR1B |= clock_select_bits;
}

void timer_start() {
	unsigned int aux_tcnt1;

	TIMSK1 &= ~_BV(TOIE1);      
	GTCCR |= _BV(PSRSYNC);   		

	old_SREG = SREG;				
	cli();						
	TCNT1 = 0;                	

	do { /* Nothing */
		old_SREG = SREG;
		cli();
		aux_tcnt1 = TCNT1;
		SREG = old_SREG;
	} while (aux_tcnt1==0); 	
	
}

void timer_restart() {
	timer_start();
}
void timer_pwm(char pin, int duty, long microseconds) {
  if(microseconds > 0) 
	  timer_set_period(microseconds);
  if(pin == PD5) {
    DDRB |= _BV(PD5);                                   
    TCCR1A |= _BV(COM1A1); 
  }	else if(pin == PD4) {
    DDRB |= _BV(PD4);
    TCCR1A |= _BV(COM1B1);
  }
  timer_set_pwm(pin, duty);
  timer_resume();			// Lex - make sure the clock is running.  We don't want to restart the count, in case we are starting the second WGM
					// and the first one is in the middle of a cycle
}
void timer_set_interrupt(void (*isr)(), long microseconds) {
	if(microseconds > 0) 
		timer_set_period(microseconds);
	isrCallback = isr;                                       
	TIMSK1 = _BV(TOIE1);                                    
	timer_resume();
}

void timer_reset_interrupt() {
	TIMSK1 &= ~_BV(TOIE1);
}

void timer_stop() {	
	TCCR1B &= ~(_BV(CS10) | _BV(CS11) | _BV(CS12));	
}

unsigned long timer_read() {
	unsigned long tmp;				// AR amended to hold more than 65536 (could be nearly double this)
  	unsigned int tcnt1;				// AR added

	old_SREG= SREG;
  	cli();							
  	tmp = TCNT1;    					
	SREG = old_SREG;

	char scale=0;
	switch (clock_select_bits)
	{
	case 1:// no prescalse
		scale = 0;
		break;
	case 2:// x8 prescale
		scale = 3;
		break;
	case 3:// x64
		scale = 6;
		break;
	case 4:// x256
		scale = 8;
		break;
	case 5:// x1024
		scale = 10;
		break;
	}
	
	do {	// Nothing -- max delay here is ~1023 cycles.  AR modified
		old_SREG = SREG;
		cli();
		tcnt1 = TCNT1;
		SREG = old_SREG;
	} while (tcnt1==tmp); //if the timer has not ticked yet

	//if we are counting down add the top value to how far we have counted down
	tmp = ((tcnt1 > tmp) ? (tmp) : (long)(ICR1-tcnt1) + (long)ICR1);		// AR amended to add casts and reuse previous TCNT1
	return ((tmp * 1000L)/(F_CPU / 1000L)) << scale;	
}
